using Battleship.DrawningObjects;
using Battleship.MovementStrategy;

namespace Battleship
{
    public partial class Battleship : Form
    {
        /// <summary>
        /// -   
        /// </summary>
        private DrawningShip? _drawningShip;
        /// <summary>
        ///  
        /// </summary>
        private AbstractStrategy? _abstractStrategy;
        public Battleship()
        {
            InitializeComponent();
        }
        ///   
        /// </summary>
        private void Draw()
        {
            if (_drawningShip == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxBattleship.Width,
            pictureBoxBattleship.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningShip.DrawTransport(gr);
            pictureBoxBattleship.Image = bmp;
        }

        private void buttonCreate_Click(object sender, EventArgs e)
        {
            Random random = new();
            _drawningShip = new DrawningShip(random.Next(100, 300),
            random.Next(1000, 3000),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256),
            random.Next(0, 256)),
            pictureBoxBattleship.Width, pictureBoxBattleship.Height);
            _drawningShip.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void buttonCreateAdd_Click(object sender, EventArgs e)
        {
            Random random = new();
            _drawningShip = new DrawningBattleship(random.Next(100, 300),
            random.Next(1000, 3000),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256),
            random.Next(0, 256)),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256),
            random.Next(0, 256)),
            Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxBattleship.Width, pictureBoxBattleship.Height);
            _drawningShip.SetPosition(random.Next(10, 100), random.Next(10,100));
            Draw();
        }

        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawningShip == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningShip.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningShip.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningShip.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningShip.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningShip == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
               DrawningObjectShip(_drawningShip), pictureBoxBattleship.Width,
               pictureBoxBattleship.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }
    }
} 